@@ -105,8 +105,9 @@ void HelpCB(void *data)
     	        i++;
     	    }
         }
-        command = xmalloc(comlen*SIZEOF_CHAR);
-        for (i = 0, j = 0; i < len - 1; i++) {
+        command = xmalloc((comlen + 1)*SIZEOF_CHAR);
+        command[comlen] = '\0';
+        for (i = 0, j = 0; i < len; i++) {
     	    if ((help_viewer[i] == '%') && (help_viewer[i + 1] == 's')){
     	        strcpy (&command[j], URL);
     	        j += urllen;
@@ -183,7 +184,7 @@ void create_about_grtool(void *data)
 	CreateLabel(rc,
             "      Portions copyright (c) 1996-1998 Boutell.Com, Inc");
 #ifdef HAVE_LIBPDF
-	CreateLabel(rc, "PDFlib library, Copyright (c) 1997-2000 Thomas Merz");
+	CreateLabel(rc, "PDFlib library, Copyright (c) 1997-2002 Thomas Merz");
 #endif
 
 	fr = CreateFrame(about_panel, "Build info");
