@@ -36,9 +36,6 @@
 #include "defines.h"
 #include "draw.h"
 
-#define BAR_HORIZONTAL  0
-#define BAR_VERTICAL    1
-
 void drawgraph(void);
 void do_hardcopy(void);
 
